var jsutils_inspect = require("../jsutils/inspect.js");

var jsutils_isAsyncIterable = require("../jsutils/isAsyncIterable.js");

var jsutils_Path = require("../jsutils/Path.js");

var error_locatedError = require("../error/locatedError.js");

var execution_collectFields = require("./collectFields.js");

var execution_execute = require("./execute.js");

var execution_mapAsyncIterator = require("./mapAsyncIterator.js");

var execution_values = require("./values.js");

var shared = require("../shared.js");

async function createSourceEventStream(...rawArgs) {
  const args = function toNormalizedArgs(args) {
    const firstArg = args[0];
    if (firstArg && "document" in firstArg) {
      return firstArg;
    }
    return {
      schema: firstArg,
      document: args[1],
      rootValue: args[2],
      contextValue: args[3],
      variableValues: args[4],
      operationName: args[5],
      subscribeFieldResolver: args[6]
    };
  }(rawArgs);
  const {schema} = args;
  execution_execute.assertValidExecutionArguments(schema);
  const exeContext = execution_execute.buildExecutionContext(args);
  if (!("schema" in exeContext)) {
    return {
      errors: exeContext
    };
  }
  try {
    const eventStream = await async function executeSubscription(exeContext) {
      const {schema, fragments, operation, variableValues, rootValue} = exeContext;
      const rootType = schema.getSubscriptionType();
      if (null == rootType) {
        throw new shared.GraphQLError("Schema is not configured to execute subscription operation.", {
          nodes: operation
        });
      }
      const rootFields = execution_collectFields.collectFields(schema, fragments, variableValues, rootType, operation.selectionSet);
      const [responseName, fieldNodes] = [ ...rootFields.entries() ][0];
      const fieldDef = execution_execute.getFieldDef(schema, rootType, fieldNodes[0]);
      if (!fieldDef) {
        throw new shared.GraphQLError(`The subscription field "${fieldNodes[0].name.value}" is not defined.`, {
          nodes: fieldNodes
        });
      }
      const path = jsutils_Path.addPath(void 0, responseName, rootType.name);
      const info = execution_execute.buildResolveInfo(exeContext, fieldDef, fieldNodes, rootType, path);
      try {
        var _fieldDef$subscribe;
        const args = execution_values.getArgumentValues(fieldDef, fieldNodes[0], variableValues);
        const contextValue = exeContext.contextValue;
        const resolveFn = null !== (_fieldDef$subscribe = fieldDef.subscribe) && void 0 !== _fieldDef$subscribe ? _fieldDef$subscribe : exeContext.subscribeFieldResolver;
        const eventStream = await resolveFn(rootValue, args, contextValue, info);
        if (eventStream instanceof Error) {
          throw eventStream;
        }
        return eventStream;
      } catch (error) {
        throw error_locatedError.locatedError(error, fieldNodes, jsutils_Path.pathToArray(path));
      }
    }(exeContext);
    if (!jsutils_isAsyncIterable.isAsyncIterable(eventStream)) {
      throw new Error(`Subscription field must return Async Iterable. Received: ${jsutils_inspect.inspect(eventStream)}.`);
    }
    return eventStream;
  } catch (error) {
    if (error instanceof shared.GraphQLError) {
      return {
        errors: [ error ]
      };
    }
    throw error;
  }
}

exports.createSourceEventStream = createSourceEventStream;

exports.subscribe = async function subscribe(args) {
  const resultOrStream = await createSourceEventStream(args);
  if (!jsutils_isAsyncIterable.isAsyncIterable(resultOrStream)) {
    return resultOrStream;
  }
  return execution_mapAsyncIterator.mapAsyncIterator(resultOrStream, (payload => execution_execute.execute({
    ...args,
    rootValue: payload
  })));
};
