import { e, h } from "../shared.mjs";

export { visit } from "../shared.mjs";

function getEnterLeaveForKind(visitor, kind) {
  if ("object" == typeof visitor[kind]) {
    return visitor[kind];
  }
  return {
    enter: visitor[kind] || visitor.enter,
    leave: visitor.leave
  };
}

function getVisitFn(visitor, kind, isLeaving) {
  const {enter, leave} = getEnterLeaveForKind(visitor, kind);
  return isLeaving ? leave : enter;
}

function visitInParallel(visitors) {
  const skipping = new Array(visitors.length).fill(null);
  const mergedVisitor = {};
  for (const kindName in e) {
    const kind = e[kindName];
    let hasVisitor = !1;
    const enterList = new Array(visitors.length).fill();
    const leaveList = new Array(visitors.length).fill();
    for (let i = 0; i < visitors.length; ++i) {
      const {enter, leave} = getEnterLeaveForKind(visitors[i], kind);
      hasVisitor = hasVisitor || null != enter || null != leave;
      enterList[i] = enter;
      leaveList[i] = leave;
    }
    if (hasVisitor) {
      mergedVisitor[kind] = {
        enter(...args) {
          const node = args[0];
          for (let i = 0; i < visitors.length; i++) {
            if (!skipping[i]) {
              const result = enterList[i] && enterList[i].apply(visitors[i], args);
              if (!1 === result) {
                skipping[i] = node;
              } else if (result === h) {
                skipping[i] = h;
              } else if (void 0 !== result) {
                return result;
              }
            }
          }
        },
        leave(...args) {
          const node = args[0];
          for (let i = 0; i < visitors.length; i++) {
            if (!skipping[i]) {
              const result = leaveList[i] && leaveList[i].apply(visitors[i], args);
              if (result === h) {
                skipping[i] = h;
              } else if (void 0 !== result && !1 !== result) {
                return result;
              }
            } else if (skipping[i] === node) {
              skipping[i] = null;
            }
          }
        }
      };
    }
  }
  return mergedVisitor;
}

export { h as BREAK, e as Kind, getEnterLeaveForKind, getVisitFn, visitInParallel };
