import { GraphQLError } from "../shared.mjs";

function getOperationRootType(schema, operation) {
  if ("query" === operation.operation) {
    const queryType = schema.getQueryType();
    if (!queryType) {
      throw new GraphQLError("Schema does not define the required query root type.", {
        nodes: operation
      });
    }
    return queryType;
  }
  if ("mutation" === operation.operation) {
    const mutationType = schema.getMutationType();
    if (!mutationType) {
      throw new GraphQLError("Schema is not configured for mutations.", {
        nodes: operation
      });
    }
    return mutationType;
  }
  if ("subscription" === operation.operation) {
    const subscriptionType = schema.getSubscriptionType();
    if (!subscriptionType) {
      throw new GraphQLError("Schema is not configured for subscriptions.", {
        nodes: operation
      });
    }
    return subscriptionType;
  }
  throw new GraphQLError("Can only have query, mutation and subscription operations.", {
    nodes: operation
  });
}

export { getOperationRootType };
