var type_definition = require("../type/definition.js");

exports.doTypesOverlap = function doTypesOverlap(schema, typeA, typeB) {
  if (typeA === typeB) {
    return !0;
  }
  if (type_definition.isAbstractType(typeA)) {
    if (type_definition.isAbstractType(typeB)) {
      return schema.getPossibleTypes(typeA).some((type => schema.isSubType(typeB, type)));
    }
    return schema.isSubType(typeA, typeB);
  }
  if (type_definition.isAbstractType(typeB)) {
    return schema.isSubType(typeB, typeA);
  }
  return !1;
};

exports.isEqualType = function isEqualType(typeA, typeB) {
  if (typeA === typeB) {
    return !0;
  }
  if (type_definition.isNonNullType(typeA) && type_definition.isNonNullType(typeB)) {
    return isEqualType(typeA.ofType, typeB.ofType);
  }
  if (type_definition.isListType(typeA) && type_definition.isListType(typeB)) {
    return isEqualType(typeA.ofType, typeB.ofType);
  }
  return !1;
};

exports.isTypeSubTypeOf = function isTypeSubTypeOf(schema, maybeSubType, superType) {
  if (maybeSubType === superType) {
    return !0;
  }
  if (type_definition.isNonNullType(superType)) {
    if (type_definition.isNonNullType(maybeSubType)) {
      return isTypeSubTypeOf(schema, maybeSubType.ofType, superType.ofType);
    }
    return !1;
  }
  if (type_definition.isNonNullType(maybeSubType)) {
    return isTypeSubTypeOf(schema, maybeSubType.ofType, superType);
  }
  if (type_definition.isListType(superType)) {
    if (type_definition.isListType(maybeSubType)) {
      return isTypeSubTypeOf(schema, maybeSubType.ofType, superType.ofType);
    }
    return !1;
  }
  if (type_definition.isListType(maybeSubType)) {
    return !1;
  }
  return type_definition.isAbstractType(superType) && (type_definition.isInterfaceType(maybeSubType) || type_definition.isObjectType(maybeSubType)) && schema.isSubType(superType, maybeSubType);
};
