import { keyValMap } from "../jsutils/keyValMap.mjs";

import { e } from "../shared.mjs";

function valueFromASTUntyped(valueNode, variables) {
  switch (valueNode.kind) {
   case e.NULL:
    return null;

   case e.INT:
    return parseInt(valueNode.value, 10);

   case e.FLOAT:
    return parseFloat(valueNode.value);

   case e.STRING:
   case e.ENUM:
   case e.BOOLEAN:
    return valueNode.value;

   case e.LIST:
    return valueNode.values.map((node => valueFromASTUntyped(node, variables)));

   case e.OBJECT:
    return keyValMap(valueNode.fields, (field => field.name.value), (field => valueFromASTUntyped(field.value, variables)));

   case e.VARIABLE:
    return null == variables ? void 0 : variables[valueNode.name.value];
  }
}

export { valueFromASTUntyped };
