var jsutils_invariant = require("../../jsutils/invariant.js");

var shared = require("../../shared.js");

exports.UniqueInputFieldNamesRule = function UniqueInputFieldNamesRule(context) {
  const knownNameStack = [];
  let knownNames = Object.create(null);
  return {
    ObjectValue: {
      enter() {
        knownNameStack.push(knownNames);
        knownNames = Object.create(null);
      },
      leave() {
        const prevKnownNames = knownNameStack.pop();
        prevKnownNames || jsutils_invariant.invariant(!1);
        knownNames = prevKnownNames;
      }
    },
    ObjectField(node) {
      const fieldName = node.name.value;
      if (knownNames[fieldName]) {
        context.reportError(new shared.GraphQLError(`There can be only one input field named "${fieldName}".`, {
          nodes: [ knownNames[fieldName], node.name ]
        }));
      } else {
        knownNames[fieldName] = node.name;
      }
    }
  };
};
