var language_visitor = require("../language/visitor.js");

var type_validate = require("../type/validate.js");

var utilities_TypeInfo = require("../utilities/TypeInfo.js");

var validation_specifiedRules = require("./specifiedRules.js");

var validation_ValidationContext = require("./ValidationContext.js");

var shared = require("../shared.js");

function validateSDL(documentAST, schemaToExtend, rules = validation_specifiedRules.specifiedSDLRules) {
  const errors = [];
  const context = new validation_ValidationContext.SDLValidationContext(documentAST, schemaToExtend, (error => {
    errors.push(error);
  }));
  const visitors = rules.map((rule => rule(context)));
  shared.visit(documentAST, language_visitor.visitInParallel(visitors));
  return errors;
}

exports.assertValidSDL = function assertValidSDL(documentAST) {
  const errors = validateSDL(documentAST);
  if (0 !== errors.length) {
    throw new Error(errors.map((error => error.message)).join("\n\n"));
  }
};

exports.assertValidSDLExtension = function assertValidSDLExtension(documentAST, schema) {
  const errors = validateSDL(documentAST, schema);
  if (0 !== errors.length) {
    throw new Error(errors.map((error => error.message)).join("\n\n"));
  }
};

exports.validate = function validate(schema, documentAST, rules = validation_specifiedRules.specifiedRules, options, typeInfo = new utilities_TypeInfo.TypeInfo(schema)) {
  var _options$maxErrors;
  const maxErrors = null !== (_options$maxErrors = null == options ? void 0 : options.maxErrors) && void 0 !== _options$maxErrors ? _options$maxErrors : 100;
  type_validate.assertValidSchema(schema);
  const abortObj = {};
  const errors = [];
  const context = new validation_ValidationContext.ValidationContext(schema, documentAST, typeInfo, (error => {
    if (errors.length >= maxErrors) {
      errors.push(new shared.GraphQLError("Too many validation errors, error limit reached. Validation aborted."));
      throw abortObj;
    }
    errors.push(error);
  }));
  const visitor = language_visitor.visitInParallel(rules.map((rule => rule(context))));
  try {
    shared.visit(documentAST, utilities_TypeInfo.visitWithTypeInfo(typeInfo, visitor));
  } catch (e) {
    if (e !== abortObj) {
      throw e;
    }
  }
  return errors;
};
